/* 
 * Author: Chris @ PyroElectro.com
 * Date: January 10, 2013
 *
 * Description:
 * This firmware is meant for a PIC18LF4520. The role of the PIC is to
 * receive 9600 baud serial commands from a transmitter. The commands
 * will be ASCII '0' to '8' representing what speed a motor should be
 * controlled out of the PWM CCP1 module. 8 distinct speeds + completely
 * off are seen in a big switch statement in the switch statement. A
 * hardware interrupt is used for capturing the serial input command.
 *
 * Project Page:
 * http://www.pyroelectro.com/tutorials/wireless_ir_motor_control/
 */

#include <p18f4520.h>
#include <delays.h>
#include <usart.h>
#include <timers.h>
#include <pwm.h>

#pragma     config OSC = HS
#pragma     config WDT = OFF
#pragma     config LVP = OFF

#define     speed_8     1
#define     speed_7     48
#define     speed_6     96
#define     speed_5     144
#define     speed_4     192
#define     speed_3     240
#define     speed_2     288
#define     speed_1     336
#define     speed_0     1000

void InterruptHandlerHigh (void);

unsigned char data;
unsigned int cnt;

void main(void){
unsigned char period = 0xFF;

TRISD = 0x00;
PORTD = 0x00;

    /*
    0b00 = Prescalar x 1
    0b01 = Prescalar x 4
    0b10 = Prescalar x 16
    */

    T2CONbits.T2CKPS0 = 0;
    T2CONbits.T2CKPS1 = 0;

    // PWM Frequency = [(period ) + 1] x 4 x TOSC x TMR2 prescaler
    // Tosc = 20 MHz
    // TMR2 Prescalar = 1
    // Period = 128
    // PWM Frequency = (256) x 4 x (1/20,000,000) x 1 = 19.5 KHz
  OpenPWM1( period );

    //Motor Initially Off
  SetDCPWM1( speed_0 );

	//Setup Interrupts
RCON 	= 0b00000000;
INTCON 	= 0b11000000;

  // configure USART
  OpenUSART( USART_TX_INT_OFF  &
             USART_RX_INT_ON  &
             USART_ASYNCH_MODE &
             USART_EIGHT_BIT   &
             USART_CONT_RX     &
             USART_BRGH_LOW, 64);

Delay10KTCYx(1);

	while(1){
           
            
    switch(data){
            case '0' :
                PORTD = 0x00;
                SetDCPWM1( speed_0 );
                break;
            case '1' :
                PORTD = 0x80;
                SetDCPWM1( speed_1 );
                break;
            case '2' :
                PORTD = 0xC0;
                SetDCPWM1( speed_2 );
                break;
            case '3' :
                PORTD = 0xE0;
                SetDCPWM1( speed_3 );
                break;
            case '4' :
                PORTD = 0xF0;
                SetDCPWM1( speed_4 );
                break;
            case '5' :
                PORTD = 0xF8;
                SetDCPWM1( speed_5 );
                break;
            case '6' :
                PORTD = 0xFC;
                SetDCPWM1( speed_6 );
                break;
            case '7' :
                PORTD = 0xFE;
                SetDCPWM1( speed_7 );
                break;
            case '8' :
                PORTD = 0xFF;
                SetDCPWM1( speed_8 );
                break;
            default  :
                PORTD = 0x00;
                break;
       }
        
            Delay10KTCYx(10);
	}

CloseUSART();
}

//**BEGIN INTERRUPT CONTROL**
#pragma code InterruptVectorHigh = 0x08		//interrupt pointer address (0x08 high priority)
void InterruptVectorHigh (void)
{
        _asm						//assembly code starts
        goto InterruptHandlerHigh	//interrupt control
        _endasm						//assembly code ends
}

#pragma code
#pragma interrupt InterruptHandlerHigh		//end interrupt control

void InterruptHandlerHigh()		// Declaration of InterruptHandler
{
        if(PIR1bits.RCIF){
			data = ReadUSART();                        
			PIR1bits.RCIF = 0;	//Clear RCIF Flag
        }
    INTCONbits.GIE = 1;						//Re-enable all interrupts
return;

}